//+------------------------------------------------------------------+
//|                                                         VWAP.mq5 |
//|                        Copyright 2020, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2020, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"

enum price_METHOD
  {
   Close,
   Open,
   High,
   Low,
   Median,  // Median price (HL/2)
   Typical, // Typical price (HLC/3)
   Weighted // Weighted Close (HLCC/4)
  };
//--- input parameters
input string            vwapID    = "01";       // VWAP ID   ( must be unique )
input price_METHOD      Method    = Typical;    // price Calculation Method
input color             vwapColor = Fuchsia;    // VWAP Color
input int               arrowSize = 3;          // Arrow Size
input ENUM_ARROW_ANCHOR Anchor    = ANCHOR_TOP; // Arrow Anchor Point

input int      InpTakeProfitPts = 100;
input int      InpStopLossPts   = 100;
input double   InpOrderSize     = 0.01;
input string   InpTradeComment  = __FILE__;
input int      InpMagicNumber   = 20000;

double         TakeProfit;
double         StopLoss;
//---buffer identification
const string   IndicatorName = "Anchor VWAP";
const int      IndexVWAP     = 0;
int            Handle;
double         BufferVWAP[];

#include <Trade\Trade.mqh>
CTrade   *Trade;

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
  Trade = new CTrade();
  Trade.SetExpertMagicNumber(InpMagicNumber);
  
  double point = SymbolInfoDouble(Symbol(), SYMBOL_POINT);
  TakeProfit   = InpTakeProfitPts * point;
  StopLoss     = InpStopLossPts * point;

  Handle = iCustom(Symbol(), 0, IndicatorName, vwapID, Method, 
                  vwapColor, arrowSize, Anchor);
                
  if (Handle==INVALID_HANDLE) 
  {
      PrintFormat("Error %i", GetLastError());
      return(INIT_FAILED);
  }
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
  IndicatorRelease(Handle);
  delete Trade; 
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   ArraySetAsSeries(BufferVWAP, true);
   if (CopyBuffer(Handle, IndexVWAP, 0, 1, BufferVWAP) < 0) {Print("CopyBufferMA1 error =",GetLastError());}
   
   MqlTick Latest_price; // Structure to get the latest prices      
   SymbolInfoTick(Symbol() ,Latest_price); // Assign current prices to structure 

// The BID price.
   static double dBid_price; 

// The ASK price.
   static double dAsk_price; 

   dBid_price = Latest_price.bid;  // Current Bid price.
   dAsk_price = Latest_price.ask;  // Current Ask price.
   
   double  currentVWAP = BufferVWAP[0];
   
// Execute strategy
   bool  buyCondition  = (dBid_price == currentVWAP);
   bool  sellCondition = (dAsk_price == currentVWAP);
   
   if (buyCondition) {
      OrderOpen(ORDER_TYPE_BUY);
   } else
   if (sellCondition) {
      OrderOpen(ORDER_TYPE_SELL);
   }
   return;
   

  }
  
bool OrderOpen(ENUM_ORDER_TYPE orderType)
{
  double price;
  double stopLossPrice;
  double takeProfitPrice;
  
  if (orderType==ORDER_TYPE_BUY)
  {
      price            =   NormalizeDouble(SymbolInfoDouble(Symbol(), SYMBOL_ASK), Digits());
      stopLossPrice    =   NormalizeDouble(price - StopLoss, Digits()) ;
      takeProfitPrice  =   NormalizeDouble(price + TakeProfit, Digits());
  } else if (orderType==ORDER_TYPE_SELL)
  {
      price            =   NormalizeDouble(SymbolInfoDouble(Symbol(), SYMBOL_BID), Digits());
      stopLossPrice    =   NormalizeDouble(price + StopLoss, Digits()) ;
      takeProfitPrice  =   NormalizeDouble(price - TakeProfit, Digits());
  } else return(false);
  
  Trade.PositionOpen(Symbol(), orderType, InpOrderSize, price, stopLossPrice, takeProfitPrice, InpTradeComment);

  return(true);
 
}
//+------------------------------------------------------------------+
